//******************************************************************************
//! @file       cc1200_rx_sniff_mode_rx.c
//! @brief      This program demonstrates Rx Sniff Mode for BOTH the 32.768 kbps
//              setting in S2-mode and C2-meter (50kbps with 2GFSK) of wM-Bus protocol.
//              The transmitter sends the EN13757-4 example packet (S2 or C2) every time a button is pushed and
//              the receiver implements RX Sniff Mode to reduce the current
//              consumption.
//              DN511 (http://www.ti.com/lit/swra428) explains how the RX Sniff related register
//              settings are found.
//
//
//  Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of Texas Instruments Incorporated nor the names of
//      its contributors may be used to endorse or promote products derived
//      from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//*****************************************************************************/


/*******************************************************************************
* INCLUDES
*/
#include "msp430.h"
#include "lcd_dogm128_6.h"
#include "hal_spi_rf_trxeb.h"
#include "cc120x_spi.h"
#include "stdlib.h"
#include "bsp.h"
#include "bsp_key.h"
#include "io_pin_int.h"
#include "bsp_led.h"
#include "cc1200_rx_sniff_mode_reg_config.h"
#include <string.h>

/*******************************************************************************
* DEFINES
*/
#define ISR_ACTION_REQUIRED     1
#define ISR_IDLE                0

#define GPIO3                   0x04
#define GPIO2                   0x08
#define GPIO0                   0x80

#define PKTLEN                  20 // 1 < PKTLEN < 126 , 20 Bytes for Payload 


#define S2mode               1
#define C2other              2
char wMBusPacketS2[]= {0x0F, 0x44, 0xAE, 0x0C, 0x78, 0x56, 0x34, 0x12, 0x01, 0x07, 0x44, 0x47, 0x78, 0x0B, 0x13, 0x43, 0x65, 0x87, 0x1E, 0x6D}; //S2-Mode
char wMBusPacketC2other[]= {0x14,0x44, 0xAE, 0x0C, 0x78, 0x56, 0x34, 0x12, 0x01, 0x07, 0x8C, 0x20, 0x27, 0x78, 0x0B, 0x13, 0x43, 0x65, 0x87, 0x7A, 0xC5}; //S2-Mode
/*******************************************************************************
* LOCAL VARIABLES
*/
static uint8  packetSemaphore = ISR_IDLE;
static uint32 packetCounter = 0;
static uint8  wmbusMode = 1;
static uint8  buttonPressed = 0x00;

/*******************************************************************************
* STATIC FUNCTIONS
*/
static void initMCU(void);
static void registerConfig(uint8 config);
static void runRX(void);
static void radioRxISR(void);
static void updateLcd(void);
static void calibrateRCOsc(void);
static void welcomeScreen(void);

/*******************************************************************************
*   @fn         main
*
*   @brief      Runs the main routine
*
*   @param      none
*
*   @return     none
*/
void main(void) {
  uint8 reg;
    // Initialize MCU and peripherals
    initMCU();
     
  // Put radio to sleep
  trxSpiCmdStrobe(CC120X_SPWD);
  
  welcomeScreen();
  
  // Wait for button push
  while(!buttonPressed){
        buttonPressed = bspKeyPushed(BSP_KEY_ALL);
      }
      if(buttonPressed == BSP_KEY_RIGHT){
        wmbusMode = C2other;
      }
      else {
        wmbusMode = S2mode;
      }
  buttonPressed = 0;
  
  // Write radio registers
  registerConfig(wmbusMode);
   
 /*
        // Mask register bit fields and write new values
    reg = 0x62; //NO invert, MANCH on

    // Write new register value
    cc120xSpiWriteReg(CC120X_MDMCFG1, &reg,1);
*/
 // Enter runRX, never coming back
    runRX();
}
//bis hier code abgeglichen mit Martin's RX

/*******************************************************************************
*   @fn         runRX
*
*   @brief      Puts radio in RX Sniff Mode and waits for wM-Bus S2-mode packets. A packet
*               counter is incremented for each packet received and the LCD is
*               updated
*               Packet Examples from EN13757-4:2012 
*   Packet S2:  0F 44 AE 0C 78 56 34 12 01 07 44 47 78 0B 13 43 65 87 1E 6D     (20 Bytes)
*   Packet C2:  14 44 AE 0C 78 56 34 12 01 07 8C 20 27 78 0B 13 43 65 87 7A C5  (21 Bytes)
*
*   @param      none
*
*   @return     none
*/
static void runRX(void) {

    uint8 rxBuffer[128] = {0};
    uint8 rxBytes;
    uint8 compareLimit = PKTLEN; //Length of S2-mode EN13757-4 packet
    uint8 marcState;
    int8 result = 1;

    // Connect ISR function to GPIO2
    ioPinIntRegister(IO_PIN_PORT_1, GPIO2, &radioRxISR);

    // Interrupt on falling edge
    ioPinIntTypeSet(IO_PIN_PORT_1, GPIO2, IO_PIN_FALLING_EDGE);              //IO_PIN_RISING_EDGE);

    // Update LCD
    updateLcd();

      // Calibrate radio
    trxSpiCmdStrobe(CC120X_SCAL);

    // Wait for calibration to be done (radio back in IDLE state)
    do {
        cc120xSpiReadReg(CC120X_MARCSTATE, &marcState, 1);
    } while (marcState != 0x41);

        
        // Calibrate the RCOSC
    calibrateRCOsc();
          
    // Clear ISR flag
    ioPinIntClear(IO_PIN_PORT_1, GPIO2);

    // Enable interrupt
    ioPinIntEnable(IO_PIN_PORT_1, GPIO2);
    
    // Infinite loop
    
    while(TRUE) {

        // Set radio in RX Sniff Mode
        trxSpiCmdStrobe(CC120X_SWOR);

        // Wait for packet to be received
        while(packetSemaphore != ISR_ACTION_REQUIRED);

        // Clear semaphore flag
        packetSemaphore = ISR_IDLE;

        // Read number of bytes in RX FIFO
       cc120xSpiReadReg(CC120X_NUM_RXBYTES, &rxBytes, 1);

        // Read all the bytes in the RX FIFO
        cc120xSpiReadRxFifo(rxBuffer, rxBytes);
        //packetCounter++;
               
        if (wmbusMode == S2mode) {
        
          // check if RX packet content is as expected
        result = strncmp((char const*)rxBuffer, (char const*)wMBusPacketS2, compareLimit);      //20 Bytes packet
        }
        else {
          result = strncmp((char const*)rxBuffer, (char const*)wMBusPacketC2other, compareLimit+1); //21 Bytes packet
        }
        if (result == 0) {
          packetCounter++;
          result = 1;
        }
          // Update LCD
          updateLcd();       

    }
}


/*******************************************************************************
*   @fn         calibrateRcOsc
*
*   @brief      Calibrates the RC oscillator used for the eWOR timer. When this
*               function is called, WOR_CFG0.RC_PD must be 0
*
*   @param      none
*
*   @return     none
*/
static void calibrateRCOsc(void) {

    uint8 temp;

    // Read current register value
    cc120xSpiReadReg(CC120X_WOR_CFG0, &temp,1);

    // Mask register bit fields and write new values
    temp = (temp & 0xF9) | (0x02 << 1);

    // Write new register value
    cc120xSpiWriteReg(CC120X_WOR_CFG0, &temp,1);

    // Strobe IDLE to calibrate the RCOSC
    trxSpiCmdStrobe(CC120X_SIDLE);

    // Disable RC calibration
    temp = (temp & 0xF9) | (0x00 << 1);
    cc120xSpiWriteReg(CC120X_WOR_CFG0, &temp, 1);
}


/*******************************************************************************
*   @fn         radioRxISR
*
*   @brief      ISR for packet handling in RX. Sets packet semaphore
*               and clears interrupt flag
*
*   @param      none
*
*   @return     none
*/
static void radioRxISR(void) {

    // Set packet semaphore
    packetSemaphore = ISR_ACTION_REQUIRED;

    // Clear ISR flag
    ioPinIntClear(IO_PIN_PORT_1, GPIO2);
}
//bis hier verglichen = OK

/*******************************************************************************
*   @fn         initMCU
*
*   @brief      Initialize MCU and board peripherals
*
*   @param      none
*
*   @return     none
*/
static void initMCU(void) {

    // Init clocks and I/O
    bspInit(BSP_SYS_CLK_8MHZ);

    // Init LEDs
    bspLedInit();

    // Init buttons
    bspKeyInit(BSP_KEY_MODE_POLL);

    // Initialize SPI interface to LCD (shared with SPI flash)
    bspIoSpiInit(BSP_FLASH_LCD_SPI, BSP_FLASH_LCD_SPI_SPD);

    // Init LCD
    lcdInit();

    // Instantiate transceiver RF SPI interface to SCLK ~ 4 MHz
    // Input parameter is clockDivider
    // SCLK frequency = SMCLK/clockDivider
    trxRfSpiInterfaceInit(2);

    // Enable global interrupt
    _BIS_SR(GIE);
}
//bis hier verglichen = OK

/*******************************************************************************
*   @fn         registerConfig
*
*   @brief      Write register settings as given by SmartRF Studio found in
*               cc1200_rx_sniff_mode_reg_config.h
*
*   @param      none
*
*   @return     none
*/
static void registerConfig(uint8 config) {

    uint8 writeByte;

    // Reset radio
    trxSpiCmdStrobe(CC120X_SRES);
  
  if( config == S2mode){
    // Write S2-mode registers to radio
    for(uint16 i = 0;
        i < (sizeof(preferredSettingsS2)/sizeof(registerSetting_t)); i++) {
          writeByte = preferredSettingsS2[i].data;
          cc120xSpiWriteReg(preferredSettingsS2[i].addr, &writeByte, 1);
        }
  }
  else{
    // Write C2other mode registers to radio
    for(uint16 i = 0;
        i < (sizeof(preferredSettingsC2other)/sizeof(registerSetting_t)); i++) {
          writeByte = preferredSettingsC2other[i].data;
          cc120xSpiWriteReg(preferredSettingsC2other[i].addr, &writeByte, 1);
        }
  }    
}

//bis hier verglichen = OK

/*******************************************************************************
*   @fn         updateLcd
*
*   @brief      Updates LCD buffer and sends buffer to LCD module
*
*   @param      none
*
*   @return     none
*/
static void updateLcd(void) {

  // Update LDC buffer and send to screen
  lcdBufferClear(0);
  lcdBufferPrintString(0, "RX Sniff S2 or C2", 0, eLcdPage0);
  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 7);
  lcdBufferPrintString(0, "Received OK:", 0, eLcdPage3);
  lcdBufferPrintInt(0, packetCounter, 70, eLcdPage4);
  if( wmbusMode == S2mode){
    lcdBufferPrintString(0, " RX, wM-Bus S2mode", 0, eLcdPage7);
  }
  else if( wmbusMode == C2other){
    lcdBufferPrintString(0, " RX, wM-Bus C2other", 0, eLcdPage7);
  }
  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 55);
  lcdBufferInvertPage(0, 0, LCD_COLS, eLcdPage7);
  lcdSendBuffer(0);
    
    
}

/*******************************************************************************
*   @fn         welcomeScreen
*
*   @brief      Updates LCD buffer and sends buffer to LCD module
*
*   @param      none
*
*   @return     none
*/
static void welcomeScreen(void) {
  
  // Update LDC buffer and send to screen
  lcdBufferClear(0);
  lcdBufferPrintString(0, "RX Sniff Mode", 0, eLcdPage0);
  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 7);
  lcdBufferPrintString(0, "Choose mode:", 0, eLcdPage3);
  lcdBufferPrintString(0, "Left:  WMBUS S2", 0, eLcdPage4);
  lcdBufferPrintString(0, "Right: WMBUS C2other", 0, eLcdPage5);
  lcdBufferPrintString(0, "RX, -110dBm", 10, eLcdPage7);  
  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 55);
  lcdBufferInvertPage(0, 0, LCD_COLS, eLcdPage7);
  lcdSendBuffer(0);
}  
