//******************************************************************************
//! @file       cc1200_rx_sniff_mode_tx.c
//! @brief      This program demonstrates Rx Sniff Mode for the 38.4 kbps
//              setting available in SmartRf Studio.
//              The transmitter sends a packet every time a button is pushed and
//              the receiver implements RX Sniff Mode to reduce the current
//              consumption.
//              DN511 (http://www.ti.com/lit/swra428) explains how the register
//              settings are found.
//
//  Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//      Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//      Neither the name of Texas Instruments Incorporated nor the names of
//      its contributors may be used to endorse or promote products derived
//      from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//*****************************************************************************/


/******************************************************************************
* INCLUDES
*/
#include "msp430.h"
#include "lcd_dogm128_6.h"
#include "hal_spi_rf_trxeb.h"
#include "cc120x_spi.h"
#include "stdlib.h"
#include "bsp.h"
#include "bsp_key.h"
#include "io_pin_int.h"
#include "bsp_led.h"
#include "cc1200_rx_sniff_mode_reg_config.h"


/*******************************************************************************
* DEFINES
*/
#define ISR_ACTION_REQUIRED     1
#define ISR_IDLE                0
#define PKTLEN                  20 // 1 < PKTLEN < 126

#define GPIO3                   0x04
#define GPIO2                   0x08
#define GPIO0                   0x80

#define PACKET_BURST            250
#define PACKET_SINGLE           1
#define WAIT_COUNTER            2500
static uint32 counter = WAIT_COUNTER;

//wMBusPacketS2[]= {0x0F, 0x44, 0xAE, 0x0C, 0x78, 0x56, 0x34, 0x12, 0x01, 0x07, 0x44, 0x47, 0x78, 0x0B, 0x13, 0x43, 0x65, 0x87, 0x1E, 0x6D, 0xFF}; //S2-Mode
char wMBusPacketS2[]= {0x0F, 0x44, 0xAE, 0x0C, 0x78, 0x56, 0x34, 0x12, 0x01, 0x07, 0x44, 0x47, 0x78, 0x0B, 0x13, 0x43, 0x65, 0x87, 0x1E, 0x6D, 0x00}; //S2-Mode
char wMBusPacketC2other[]= {0x14,0x44, 0xAE, 0x0C, 0x78, 0x56, 0x34, 0x12, 0x01, 0x07, 0x8C, 0x20, 0x27, 0x78, 0x0B, 0x13, 0x43, 0x65, 0x87, 0x7A, 0xC5}; //C2-OTHER mode
/*******************************************************************************
* LOCAL VARIABLES
*/
static uint8  packetSemaphore;
static uint32 packetCounter = 0;
static uint8  buttonPressed = 0x00;
static uint8  wmbusMode = 1;
#define S2mode               1
#define C2other              2

/*******************************************************************************
* STATIC FUNCTIONS
*/
static void initMCU(void);
static void registerConfig(uint8 config);
static void runTX(void);
static void createPacket(uint8 randBuffer[]);
static void radioTxISR(void);
static void updateLcd(void);
static void welcomeScreen(void);

/*******************************************************************************
*   @fn         main
*
*   @brief      Runs the main routine.
*
*   @param      none
*
*   @return     none
*/
void main(void) {

    // Initialize MCU and peripherals
    initMCU();
    
     
  // Put radio to sleep
  trxSpiCmdStrobe(CC120X_SPWD);
  
  welcomeScreen();
  
  // Wait for button push
  while(!buttonPressed){
        buttonPressed = bspKeyPushed(BSP_KEY_ALL);
      }
      if(buttonPressed == BSP_KEY_RIGHT){
        wmbusMode = C2other;
      }
      else {
        wmbusMode = S2mode;
      }
  buttonPressed = 0;
  
  // Write radio registers
  registerConfig(wmbusMode);
      
   // Enter runTX, never coming back
   runTX();
    
    
}


/*******************************************************************************
*   @fn         runTX
*
*   @brief      Transmits a packet every time a button is pushed. A packet
*               counter is incremented for each packet sent and the LCD is
*               updated
*
*   @param      none
*
*   @return     none
*/
static void runTX(void) {

    static uint8 marcState;
   uint8 numPackets = 0;
    // Initialize packet buffer of size PKTLEN + 1
    uint8 txBuffer[PKTLEN + 1];

    // Connect ISR function to GPIO0
    ioPinIntRegister(IO_PIN_PORT_1, GPIO0, &radioTxISR);

    // Interrupt on falling edge
    ioPinIntTypeSet(IO_PIN_PORT_1, GPIO0, IO_PIN_FALLING_EDGE);

    // Clear ISR flag
    ioPinIntClear(IO_PIN_PORT_1, GPIO0);

    // Enable interrupt
    ioPinIntEnable(IO_PIN_PORT_1, GPIO0);

    // Update LCD
    updateLcd();

    // Calibrate radio
    trxSpiCmdStrobe(CC120X_SCAL);

    // Wait for calibration to be done (radio back in IDLE state)
    do {
        cc120xSpiReadReg(CC120X_MARCSTATE, &marcState, 1);
    } while (marcState != 0x41);

        // Infinite loop for Single or Burst
    while(TRUE) {

        // Wait for button push
     // while(!bspKeyPushed(BSP_KEY_ALL));
        
      while(!buttonPressed){
      buttonPressed = bspKeyPushed(BSP_KEY_ALL);
    }
    if(buttonPressed == BSP_KEY_LEFT){
      numPackets = PACKET_BURST;
    }
    else{
      numPackets = PACKET_SINGLE;
    }
    buttonPressed = 0;    
     // Infinite loop for Single or Burst
   
    // Create the wM-Bus S2- or C2-OTHER mode wM-Bus packet
        createPacket(txBuffer);
    
    do {  
     
               // Write packet to TX FIFO
      cc120xSpiWriteTxFifo((uint8*)txBuffer, sizeof(txBuffer));
              // intervall between packets
      while(counter--)NOP();
      counter = WAIT_COUNTER; 
      
        // Strobe TX to send packet
        trxSpiCmdStrobe(CC120X_STX);

        // Wait for packet to be sent
        while(packetSemaphore != ISR_ACTION_REQUIRED);

        // Clear semaphore flag
        packetSemaphore = ISR_IDLE;
        numPackets--;
        // Update LCD
        updateLcd();
    } while(numPackets);
    }    
}


/*******************************************************************************
* @fn          radioTxISR
*
* @brief       ISR for packet handling in TX. Sets packet semaphore
*              and clears interrupt flag
*
* @param       none
*
* @return      none
*/
static void radioTxISR(void) {

    // Set packet semaphore
    packetSemaphore = ISR_ACTION_REQUIRED;

    // Clear ISR flag
    ioPinIntClear(IO_PIN_PORT_1, GPIO0);
}


/*******************************************************************************
*   @fn         initMCU
*
*   @brief      Initialize MCU and board peripherals
*
*   @param      none
*
*   @return     none
*/
static void initMCU(void) {

    // Init clocks and I/O
    bspInit(BSP_SYS_CLK_8MHZ);

    // Init LEDs
    bspLedInit();

    // Init buttons
    bspKeyInit(BSP_KEY_MODE_POLL);

    // Initialize SPI interface to LCD (shared with SPI flash)
    bspIoSpiInit(BSP_FLASH_LCD_SPI, BSP_FLASH_LCD_SPI_SPD);

    // Init LCD
    lcdInit();

    // Instantiate transceiver RF SPI interface to SCLK ~ 4 MHz
    // Input parameter is clockDivider
    // SCLK frequency = SMCLK/clockDivider
    trxRfSpiInterfaceInit(2);

    // Enable global interrupt
    _BIS_SR(GIE);
}


/*******************************************************************************
*   @fn         registerConfig
*
*   @brief      Write register settings as given by SmartRF Studio found in
*               cc1200_rx_sniff_mode_reg_config.h
*
*   @param      none
*
*   @return     none
*/
static void registerConfig(uint8 config) {

    uint8 writeByte;
  // Reset radio
    trxSpiCmdStrobe(CC120X_SRES);
  
  if( config == S2mode){
    // Write registers to radio
    for(uint16 i = 0;
        i < (sizeof(preferredSettingsS2)/sizeof(registerSetting_t)); i++) {
          writeByte = preferredSettingsS2[i].data;
          cc120xSpiWriteReg(preferredSettingsS2[i].addr, &writeByte, 1);
        }
  }
  else{
    // Write registers to radio
    for(uint16 i = 0;
        i < (sizeof(preferredSettingsC2other)/sizeof(registerSetting_t)); i++) {
          writeByte = preferredSettingsC2other[i].data;
          cc120xSpiWriteReg(preferredSettingsC2other[i].addr, &writeByte, 1);
        }
  }    
}


/*******************************************************************************
*   @fn         createPacket
*
*   @brief      This function is called before a packet is transmitted. It fills
*               the txBuffer with a packet consisting of a length byte, two
*               bytes packet counter and n random bytes
*
*               The packet format is as follows:
*               |--------------------------------------------------------------|
*               |           |           |           |         |       |        |
*               | Data 0    | Data 1    | Data 2    | ....... |.......| Data19 |
*               |           |           |           |         |       |        |
*               |--------------------------------------------------------------|
*                txBuffer[0] txBuffer[1] txBuffer[2]  ......... txBuffer[PKTLEN]
*
0F 44 AE 0C 78 56 34 12 01 07 44 47 78 0B 13 43 65 87 1E 6D + 00 (which gives the S2-mode postamble)

*   @param       Pointer to start of txBuffer
*
*   @return      none
*/
static void createPacket(uint8 txBuffer[]) {

  if ( wmbusMode == S2mode ) {
    txBuffer[0] = 0x0F;                         // Data 0
    txBuffer[1] = 0x44; 
    txBuffer[2] = 0xAE;                         // Data 2
    txBuffer[3] = 0x0C; 
    
    txBuffer[4] = 0x78;                         // Data 4
    txBuffer[5] = 0x56; 
    txBuffer[6] = 0x34;                         // Data 6
    txBuffer[7] = 0x12; 
    
    txBuffer[8] = 0x01;                         // Data 8
    txBuffer[9] = 0x07; 
    txBuffer[10] = 0x44;                         // Data 10
    txBuffer[11] = 0x47; 
    
    txBuffer[12] = 0x78;                         // Data 12
    txBuffer[13] = 0x0B; 
    txBuffer[14] = 0x13;                         // Data 14
    txBuffer[15] = 0x43; 
    
    txBuffer[16] = 0x65;                         // Data 16
    txBuffer[17] = 0x87; 
    txBuffer[18] = 0x1E;                         // Data 18
    txBuffer[19] = 0x6D; 
    txBuffer[20] = 0;
  }
  // {0x14,0x44, 0xAE, 0x0C, 0x78, 0x56, 0x34, 0x12, 0x01, 0x07, 0x8C, 0x20, 0x27, 0x78, 0x0B, 0x13, 0x43, 0x65, 0x87, 0x7A, 0xC5}; //C2-OTHER mode
  else {
    txBuffer[0] = 0x14;                         // Data 0
    txBuffer[1] = 0x44; 
    txBuffer[2] = 0xAE;                         // Data 2
    txBuffer[3] = 0x0C; 
    
    txBuffer[4] = 0x78;                         // Data 4
    txBuffer[5] = 0x56; 
    txBuffer[6] = 0x34;                         // Data 6
    txBuffer[7] = 0x12; 
    
    txBuffer[8] = 0x01;                         // Data 8
    txBuffer[9] = 0x07;
    
    txBuffer[10] = 0x8C;                         // Data 10
    txBuffer[11] = 0x20; 
    
    txBuffer[12] = 0x27;                         // Data 12
    txBuffer[13] = 0x78; 
    txBuffer[14] = 0x0B;                         // Data 14
    txBuffer[15] = 0x13; 
    
    txBuffer[16] = 0x43;                         // Data 16
    txBuffer[17] = 0x65; 
    txBuffer[18] = 0x87;                         // Data 18
    txBuffer[19] = 0x7A; 
    txBuffer[20] = 0xC5;
    
  }
}


/*******************************************************************************
*   @fn         updateLcd
*
*   @brief      Updates LCD buffer and sends bufer to LCD module
*
*   @param      none
*
*   @return     none
*/
static void updateLcd(void) {

    // Update LDC buffer and send to screen
    lcdBufferClear(0);
    lcdBufferPrintString(0, "TX S2- / C2-OTHER", 0, eLcdPage0);
    lcdBufferSetHLine(0, 0, LCD_COLS - 1, 7);
    
    lcdBufferPrintString(0, "Sent packets:", 0, eLcdPage3);
  lcdBufferPrintInt(0, packetCounter++, 80, eLcdPage3);
  
  
      if( wmbusMode == S2mode){
        lcdBufferPrintString(0, " TX, wM-Bus S2mode", 0, eLcdPage7);
      }
      else if( wmbusMode == C2other){
        lcdBufferPrintString(0, " TX, wM-Bus C2other", 0, eLcdPage7);
      }

  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 55);
  lcdBufferInvertPage(0, 0, LCD_COLS, eLcdPage7);
  lcdSendBuffer(0);
   
}

 


/*******************************************************************************
*   @fn         welcomeScreen
*
*   @brief      Updates LCD buffer and sends buffer to LCD module
*
*   @param      none
*
*   @return     none
*/
static void welcomeScreen(void) {
  
  // Update LDC buffer and send to screen
  lcdBufferClear(0);
  lcdBufferPrintString(0, "TX in S2-/C2-OTHER Mode", 0, eLcdPage0);
  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 7);
  lcdBufferPrintString(0, "Choose mode:", 0, eLcdPage3);
  lcdBufferPrintString(0, "Left:  WMBUS S2", 0, eLcdPage4);
  lcdBufferPrintString(0, "Right: WMBUS C2other", 0, eLcdPage5);
  lcdBufferPrintString(0, "TX packets", 10, eLcdPage7);  
  lcdBufferSetHLine(0, 0, LCD_COLS - 1, 55);
  lcdBufferInvertPage(0, 0, LCD_COLS, eLcdPage7);
  lcdSendBuffer(0);
}  
